% function [scores,labels] = dice(A,B)
%   Compute Dice similarity:  2 |A inter B| / (|A| + |B|)
%   A,B - binary (or multilabel) column vectors (score is the dice metric for each column)
%   scores - dice scores for each label
%   label  - label value, sorted and associated with each scores
%
% herve.lombaert@polymtl.ca, June 2015
%
function [scores,labels] = dice(A,B)

    if nargin ~= 2;                error('d = dice(A,B)');              end;
    if ~isequal(size(A),size(B));  error('A and B : different sizes');  end;

    % Check all existing labels
    labels = unique([unique(A(:)); unique(B(:))]);
    scores = zeros(numel(labels),size(A,2));
    
    for iter_label = 1:numel(labels)
        
        % Get labels
        lbl = labels(iter_label);
    
        Ai = 1*(A==lbl);
        Bi = 1*(B==lbl);

        % Get Dice Score
        sc = 2 * sum(Ai & Bi) ./ (sum(Ai) + sum(Bi));
        
        zr = (sum(A) + sum(B));  sc(zr==0) = 0; % zero for empty sets
        
        scores(iter_label,:) = sc;
        
    end
    
end
