% h = displaymesh(X,F,C)
%   Display triangular mesh
%     X - list of vertices
%     F - list of triangular faces (indices of vertices X)
%     C - optional coloring
%     h - handle on trisurf (e.g., for setting independent transparency)
%
% herve.lombaert@gmail.com, May 2015
%
function h = displaymesh(X,F,C)

    if nargin<3;  C = 1:size(X,1);  end;
    
    % display mesh
    h = trisurf(F, X(:,1), X(:,2), X(:,3), C);  
    shading interp;  daspect([1 1 1]);  axis on;  material dull;  
    xlabel('x');  ylabel('y');  zlabel('z');
    
    % find best orthogonal view
    [~,idx] = sort(range(X(:,1:3)));
    if     idx(1) == 3, view(0,0);
    elseif idx(2) == 3, view(-90,0);
    elseif idx(3) == 3, view(0,90);
    else, view(0,0);
    end    
    
    % position light
    if numel(findobj(gca,'type','light')) == 0
        camlight
    end
    link_light;
   
    if nargout < 1;  clear h;  end;
end
