% do_multithread(0|1)
%    Global constant variable
%       Set with:    do_multithread(1);
%       Query with:  if do_multithread;  disp('multithreading');  end;
%
function o = do_multithread(i)

    if ~exist('matlabpool','file') && ~exist('gcp','file')
        if nargin>0;  warning('Matlab parallel not present');  end;
        o = 0;  
        return;
    end

    persistent varval;
    if isempty(varval);  varval = isactive;  end;
    if nargin>0;         varval = i;  end;
    activate(varval);
    if nargout==0;       return;      end;
    o = varval;
    
end

function activate(i)

    if exist('gcp','file')
        if i  % Activate
            if isempty(gcp('nocreate'));   parpool('IdleTimeout',120);  end;
        else  % Deactivate
            if ~isempty(gcp('nocreate'));  delete(gcp('nocreate'));  end;
        end
    elseif exist('matlabpool','file')
        if i  % Activate
            if matlabpool('size')==0;  matlabpool;  end;
        else  % Deactivate
            if matlabpool('size')>0;   matlabpool close;  end;
        end
    else
        warning('Matlab parallel not present');
    end
end

function o = isactive

    o = 0;
    if exist('gcp','file')
        o = ~isempty(gcp('nocreate'));
    elseif exist('matlabpool','file')
        o = (matlabpool('size')>0);
    else
        warning('Matlab parallel not present');
    end
end
