function [M1, M2, corr] = load_meshes(pathname, id1, id2)

if numel(id1) == 4 && numel(id2) == 4
    
%     id1 = '033L';
%     id2 = '017L';

    if isempty(pathname);  pathname = '/Users/hlombaer/home/data/spectral-cortical';  end;

    % mesh
    M1  = load([pathname '/meshes/smooth-0/' id1 '.mesh.mat']);
    M1.n = size(M1.X,1);
    tmp = load([pathname '/properties/' id1 '.properties.mat']);
    M1.S = tmp.Sulcaldepth;
    M1.P = zeros(M1.n,1);  [i,j] = find(tmp.Parcels);  M1.P(i) = j;  % Parcels

    M2  = load([pathname '/meshes/smooth-0/' id2 '.mesh.mat']);
    M2.n = size(M2.X,1);
    tmp = load([pathname '/properties/' id2 '.properties.mat']);
    M2.S = tmp.Sulcaldepth;
    M2.P = zeros(M2.n,1);  [i,j] = find(tmp.Parcels);  M2.P(i) = j;  % Parcels
    
    % smooth
    tmp = load([pathname '/meshes/smooth-200/' id1 '.mesh.mat']);
    M1.Xs = tmp.X;
    tmp = load([pathname '/meshes/smooth-200/' id2 '.mesh.mat']);
    M2.Xs = tmp.X;

    % FS correspondence
    tmp = load([pathname '/results/correspondence_freesurfer/' id1 '_' id2 '.mat']);
    corr12fs = tmp.Correspondence;
    tmp = load([pathname '/results/correspondence_freesurfer/' id2 '_' id1 '.mat']);
    corr.corr21fs = tmp.Correspondence;
    
    % Spectral Warp correspondence
    tmp = load([pathname '/results/correspondence_warp/' id2 '_' id1 '.mat']);
    corr.corr12wp = tmp.corr21;
    corr.corr21wp = tmp.corr12;
    
    % FOCUSR correspondence
    tmp = load([pathname '/results/correspondence_focusr/' id2 '_' id1 '.mat']);
    corr.corr12fc = tmp.corr21;
    corr.corr21fc = tmp.corr12;
    
else
    
    %% Reading meshes
%     id1 = 'OASIS-TRT-20-1';  % 3-4 72.9 (FS 77.5)
%     id2 = 'OASIS-TRT-20-2';  % 5-6

    if isempty(pathname);  pathname = '/data/datasets/Mindboggle101/processed/processed_FreeSurfer';  end;

    filename = {...
        [pathname '/' id1 '/surf/lh.white'], ...
        [pathname '/' id2 '/surf/lh.white'] ...
    };
    [M1.X,M1.F] = read_surf(filename{1});
    [M2.X,M2.F] = read_surf(filename{2});
    M1.n = size(M1.X,1);
    M2.n = size(M2.X,1);

    %% Read inflated
    filename_inflated = {...
        [pathname '/' id1 '/surf/lh.inflated'], ...
        [pathname '/' id2 '/surf/lh.inflated'], ...
    };
    if exist(filename_inflated{1},'file'); [M1.Xs,~] = read_surf(filename_inflated{1}); end;
    if exist(filename_inflated{2},'file'); [M2.Xs,~] = read_surf(filename_inflated{2}); end;

    %% Read Manual annotations
    filename_annot = {...
        [pathname '/' id1 '/label/lh.labels.DKT31.manual.annot'], ...
        [pathname '/' id2 '/label/lh.labels.DKT31.manual.annot'], ...
    };
    if exist(filename_annot{1},'file'); [~,M1.Pm,M1.Pm_info] = read_annotation(filename_annot{1}); end;
    if exist(filename_annot{2},'file'); [~,M2.Pm,M2.Pm_info] = read_annotation(filename_annot{2}); end;

    %% Read FS annotations
    filename_annotfs = {...
        [pathname '/' id1 '/label/lh.aparc.annot'], ...
        [pathname '/' id2 '/label/lh.aparc.annot'], ...
    };
    if exist(filename_annotfs{1},'file'); [~,M1.P,M1.P_info] = read_annotation(filename_annotfs{1}); end;
    if exist(filename_annotfs{2},'file'); [~,M2.P,M2.P_info] = read_annotation(filename_annotfs{2}); end;

    %% Read registered FS
    filename_regFS = {...
        [pathname '/' id1 '/surf/lh.sphere.reg'], ...
        [pathname '/' id2 '/surf/lh.sphere.reg'], ...
    };
    if exist(filename_regFS{1},'file'); [M1.Xfs,~] = read_surf(filename_regFS{1}); end;
    if exist(filename_regFS{2},'file'); [M2.Xfs,~] = read_surf(filename_regFS{2}); end;
    if exist(filename_regFS{1},'file') && exist(filename_regFS{2},'file');  [corr.corr12fs,corr.corr21fs] = match(M1.Xfs, M2.Xfs);  end;
    if ~exist('corr','var');  corr = [];  end;

    %% Read sulcal depth
    filename_sulcal = {...
        [pathname '/' id1 '/surf/lh.sulc'], ...
        [pathname '/' id2 '/surf/lh.sulc'], ...
    };
    if exist(filename_sulcal{1},'file'); [M1.S] = read_curv(filename_sulcal{1}); end;
    if exist(filename_sulcal{2},'file'); [M2.S] = read_curv(filename_sulcal{2}); end;

end


%% Compute State-of-the-art correspondence
if isfield(M1,'X') & isfield(M2,'X')
    if isfield(M1,'P') & isfield(M2,'P')
        %% FreeSurfer
        if isfield(corr,'corr12fs')
            [scores12fs,~] = dice(M1.P, M2.P(corr.corr12fs));
            disp(['(FS 1->2) Dice FS (avg+-std [min/max]%): ' num2str(100*mean(scores12fs),'%.1f') '+-' num2str(100*std(scores12fs),'%.1f') ' [' num2str(100*min(scores12fs),'%.1f') '/' num2str(100*max(scores12fs),'%.1f') ']']);
        end
        if isfield(corr,'corr21fs')
            [scores21fs,~] = dice(M1.P(corr.corr21fs), M2.P);
            disp(['(FS 2->1) Dice FS (avg+-std [min/max]%): ' num2str(100*mean(scores21fs),'%.1f') '+-' num2str(100*std(scores21fs),'%.1f') ' [' num2str(100*min(scores21fs),'%.1f') '/' num2str(100*max(scores21fs),'%.1f') ']']);
        end
        if isfield(corr,'corr12fs') & isfield(M1,'Pm') & isfield(M2,'Pm')
            [scores12fs,~] = dice(M1.Pm, M2.Pm(corr.corr12fs));
            disp(['(FS 1->2) Dice Mn (avg+-std [min/max]%): ' num2str(100*mean(scores12fs),'%.1f') '+-' num2str(100*std(scores12fs),'%.1f') ' [' num2str(100*min(scores12fs),'%.1f') '/' num2str(100*max(scores12fs),'%.1f') ']']);
        end
        if isfield(corr,'corr21fs') & isfield(M1,'Pm') & isfield(M2,'Pm')
            [scores21fs,~] = dice(M1.Pm(corr.corr21fs), M2.Pm);
            disp(['(FS 2->1) Dice Mn (avg+-std [min/max]%): ' num2str(100*mean(scores21fs),'%.1f') '+-' num2str(100*std(scores21fs),'%.1f') ' [' num2str(100*min(scores21fs),'%.1f') '/' num2str(100*max(scores21fs),'%.1f') ']']);
        end

        %% Spectral Warping (Brain Transfer)
        if isfield(corr,'corr12wp')
            [scores12wp,~] = dice(M1.P, M2.P(corr.corr12wp));
            disp(['(Wp 1->2) Dice FS (avg+-std [min/max]%): ' num2str(100*mean(scores12wp),'%.1f') '+-' num2str(100*std(scores12wp),'%.1f') ' [' num2str(100*min(scores12wp),'%.1f') '/' num2str(100*max(scores12wp),'%.1f') ']']);
        end
        if isfield(corr,'corr21wp')
            [scores21wp,~] = dice(M1.P(corr.corr21wp), M2.P);
            disp(['(Wp 2->1) Dice FS (avg+-std [min/max]%): ' num2str(100*mean(scores21wp),'%.1f') '+-' num2str(100*std(scores21wp),'%.1f') ' [' num2str(100*min(scores21wp),'%.1f') '/' num2str(100*max(scores21wp),'%.1f') ']']);
        end
        if isfield(corr,'corr12wp') & isfield(M1,'Pm') & isfield(M2,'Pm')
            [scores12wp,~] = dice(M1.P, M2.P(corr.corr12wp));
            disp(['(Wp 1->2) Dice Mn (avg+-std [min/max]%): ' num2str(100*mean(scores12wp),'%.1f') '+-' num2str(100*std(scores12wp),'%.1f') ' [' num2str(100*min(scores12wp),'%.1f') '/' num2str(100*max(scores12wp),'%.1f') ']']);
        end
        if isfield(corr,'corr21wp') & isfield(M1,'Pm') & isfield(M2,'Pm')
            [scores21wp,~] = dice(M1.P(corr.corr21wp), M2.P);
            disp(['(Wp 2->1) Dice Mn (avg+-std [min/max]%): ' num2str(100*mean(scores21wp),'%.1f') '+-' num2str(100*std(scores21wp),'%.1f') ' [' num2str(100*min(scores21wp),'%.1f') '/' num2str(100*max(scores21wp),'%.1f') ']']);
        end
        
        %% FOCUSR
        if isfield(corr,'corr12fc')
            [scores12fc,~] = dice(M1.P, M2.P(corr.corr12fc));
            disp(['(Fc 1->2) Dice FS (avg+-std [min/max]%): ' num2str(100*mean(scores12fc),'%.1f') '+-' num2str(100*std(scores12fc),'%.1f') ' [' num2str(100*min(scores12fc),'%.1f') '/' num2str(100*max(scores12fc),'%.1f') ']']);
        end
        if isfield(corr,'corr21fc')
            [scores21fc,~] = dice(M1.P(corr.corr21fc), M2.P);
            disp(['(Fc 2->1) Dice FS (avg+-std [min/max]%): ' num2str(100*mean(scores21fc),'%.1f') '+-' num2str(100*std(scores21fc),'%.1f') ' [' num2str(100*min(scores21fc),'%.1f') '/' num2str(100*max(scores21fc),'%.1f') ']']);
        end
        if isfield(corr,'corr12fc') & isfield(M1,'Pm') & isfield(M2,'Pm')
            [scores12fc,~] = dice(M1.Pm, M2.Pm(corr12fc));
            disp(['(Fc 1->2) Dice Mn (avg+-std [min/max]%): ' num2str(100*mean(scores12fc),'%.1f') '+-' num2str(100*std(scores12fc),'%.1f') ' [' num2str(100*min(scores12fc),'%.1f') '/' num2str(100*max(scores12fc),'%.1f') ']']);
        end
        if isfield(corr,'corr21fc') & isfield(M1,'Pm') & isfield(M2,'Pm')
            [scores21fc,~] = dice(M1.Pm(corr.corr21fc), M2.Pm);
            disp(['(Fc 2->1) Dice Mn (avg+-std [min/max]%): ' num2str(100*mean(scores21fc),'%.1f') '+-' num2str(100*std(scores21fc),'%.1f') ' [' num2str(100*min(scores21fc),'%.1f') '/' num2str(100*max(scores21fc),'%.1f') ']']);
        end
    end
end

end
