% mytoc
%   Stop timer and display elapsed time (set global variable : do_verbose)
%     string - timer name (default: '')
%     t - Elapsed time;
%
% herve.lombaert@gmail.com, May 2016
%
function t = mytoc(string)

    %if nargin<1;  string = getcallersname;  end;
    if nargin<1;  string = '';  end;
    
    is_multithreading = ( exist('gcp','file') && ~isempty(gcp('nocreate')) ) || ( ~exist('gcp','file') && exist('matlabpool','file') && (matlabpool('size')>0) );
    
    if ~is_multithreading
        
        % Here: t = toc;
        global gtictoc_cnt gtictoc_times gtictoc_last_strings gtictoc_report;
        if isempty(gtictoc_cnt);  warning('No mytic() called previously'); return;  end;
        t = clock; %cputime;
        dt = etime(t, gtictoc_times(gtictoc_cnt,:));

        % Prepare string
        if isempty(string);  string = gtictoc_last_strings{gtictoc_cnt};  end;
        if isempty(string);  string = getcallersname;  end;
        prestring = '*** Timing (';
        if gtictoc_cnt>1;  for i=1:(gtictoc_cnt-1); prestring = [ prestring '.' ]; end;  end;
        time_in_minutes = [];  if dt>120;  time_in_minutes = [' (' num2str(dt/60,'%.2f') ' mins)'];  end;
        string = [ prestring ') ' string ': ' num2str(dt,'%.2f') ' secs' time_in_minutes '.'];
        if do_verbose;  disp(string);  end
        
        if isempty(gtictoc_report);  gtictoc_report = [];  end;
        gtictoc_report{numel(gtictoc_report)+1} = string;

        % Decrease tictoc counter
        gtictoc_last_strings{gtictoc_cnt} = [];
        gtictoc_cnt = gtictoc_cnt - 1;
        if gtictoc_cnt<1
            gtictoc_cnt = []; gtictoc_times = []; gtictoc_last_strings = [];
            %if do_verbose;  disp('*** Timing Report');  for i=1:numel(gtictoc_report);  disp(gtictoc_report{i});  end;  end;
            gtictoc_report = [];
        end

    else
        
        % Do not overwrite global variable, instead create new ones that
        % has their scope within each parfor only
        
        % Here: t = toc;
        global g_mt_tictoc_cnt g_mt_tictoc_times g_mt_tictoc_last_strings;
        if isempty(g_mt_tictoc_cnt);  warning('No mytic() called previously'); return;  end;
        t = clock;  %cputime;
        dt = etime( t, g_mt_tictoc_times(g_mt_tictoc_cnt,:) );  %dt = t - g_mt_tictoc_times(g_mt_tictoc_cnt);

        if do_verbose
            if isempty(string);  string = g_mt_tictoc_last_strings{g_mt_tictoc_cnt};  end;
            if isempty(string);  string = getcallersname;  end;
            prestring = '*** Timing (';
            %if g_mt_tictoc_cnt>1;  prestring = [num2str(g_mt_tictoc_cnt-1) '-'];  end;
            if g_mt_tictoc_cnt>1;  for i=1:(g_mt_tictoc_cnt-1); prestring = [ prestring '.' ]; end;  end;
            time_in_minutes = [];  if dt>120;  time_in_minutes = [' (' num2str(dt/60,'%.2f') ' mins)'];  end;
            %if do_verbose;  disp(['*** ' prestring 'Elapsed ' string ': ' num2str(dt) ' secs' time_in_minutes '.']);  end;
            string = [ prestring ') ' string ': ' num2str(dt,'%.2f') ' secs' time_in_minutes '.'];
            if do_verbose;  disp(string);  end
        end

        % Decrease tictoc counter
        g_mt_tictoc_last_strings{g_mt_tictoc_cnt} = [];
        g_mt_tictoc_cnt = g_mt_tictoc_cnt - 1;
        if g_mt_tictoc_cnt<1;  g_mt_tictoc_cnt = []; g_mt_tictoc_times = [];  g_mt_tictoc_last_strings = [];  end;
    
    end
    
    if nargout < 1;  clear t;  end;
        
end

function s = getcallersname

    st = dbstack;
    if size(st,1)>2;  s = st(3).name;
    else              s = '';          end;

end
